extends Node2D

export var num = 0 setget set_num, get_num
var MAX_SPEED = 15

var posxy = Vector2(0, 0)
var inertia = Vector2(0, 0)

var color = Color(0, 0, 0)

var rotation = 0
var ROTATION_SPEED = 0.5 #Spins per second

var posz = 5
var jump_vector = Vector3(0, 0, 0)
var MAX_REGISTER_JUMP = 1
var Z_ACCEL = -35
var INITIAL_JUMP_SPEED = 12

var INITIAL_MOVE_SPEED = 5

var dead = false
var kill_anim = 0

var KILL_ANIM_DURATION = 1

var RESPAWN_ANIM_FREQ =  0.2
var time = 0

var hits = 3

var line_size = 3

func _ready():
	line_size = get_node("/root/Point3DManager").get_line_size()
	set_process(true)

func _process(delta):
	if get_node("/root/GameData").is_player_dead(num):
		dead = true
		kill_anim += delta
		update()
		if kill_anim >= KILL_ANIM_DURATION:
			get_node("/root/Waves").remove_all_waves()
			get_node("/root/GameData").next_round()
		return
	
	var direction = Vector2(0, 0)
	if (Input.is_action_pressed("user1_up") && num == 1) || \
		(Input.is_action_pressed("user2_up") && num == 2):
		direction += Vector2(0, -1)
	if (Input.is_action_pressed("user1_down") && num == 1) || \
		(Input.is_action_pressed("user2_down") && num == 2):
		direction += Vector2(0, 1)
	if (Input.is_action_pressed("user1_right") && num == 1) || \
		(Input.is_action_pressed("user2_right") && num == 2):
		direction += Vector2(1, 0)
	if (Input.is_action_pressed("user1_left") && num == 1) || \
		(Input.is_action_pressed("user2_left") && num == 2):
		direction += Vector2(-1, 0)
	
	direction = direction * INITIAL_MOVE_SPEED * delta
	
	var INERTIA_DAMPENING = 0.9
	var INERTIA_FACTOR  = 1.00
	
	if get_node("/root/GameData").is_level_icy():
		INERTIA_DAMPENING = 0.9
		INERTIA_FACTOR = 1.09
	
	direction = direction * (1.0 - INERTIA_DAMPENING) + \
		inertia * INERTIA_DAMPENING
	inertia = direction * INERTIA_FACTOR
	
	posxy += direction
	
	var griddim = get_node("/root/Point3DManager").get_grid_dimension()
	if posxy.x < 0:
		posxy.x *= -1
		inertia.x *= -1
	if posxy.y < 0:
		posxy.y *= -1
		inertia.y *= -1
	if posxy.x > griddim.x - 1:
		posxy.x = 2 * (griddim.x - 1) - posxy.x
		inertia.x *= -1
	if posxy.y > griddim.y - 1:
		posxy.y = 2 * (griddim.y - 1) - posxy.y
		inertia.y *= -1
	
	#posxy.x = min(max(posxy.x, 0), griddim.x - 1) 
	#posxy.y = min(max(posxy.y, 0), griddim.y - 1)
	
	if (Input.is_action_pressed("user1_jump") && num == 1) || \
		(Input.is_action_pressed("user2_jump") && num == 2):
		if posz < MAX_REGISTER_JUMP && jump_vector.z >= 0:
			if jump_vector.z == 0:
				get_node("/root/World/SamplePlayer").play("jumpfx", false)
			jump_vector.z = INITIAL_JUMP_SPEED
		
	var world = get_node("/root/World")
	
	if jump_vector.z > 0:	#Raising
		posz += jump_vector.z * delta
		jump_vector.z += Z_ACCEL * delta
	elif jump_vector.z <= 0 && posz > 0:	#Falling
		var delta_jump = jump_vector.z * delta
		if posz + delta_jump < 0: #Fallen
			get_node("/root/World/SamplePlayer").play("fallfx", false)
			if world.not_void(posxy):
				var wave_manager = get_node("/root/Waves")
				wave_manager.add_wave(posxy.x, posxy.y, num)
			world.set_p_jumped(num, posxy)
			jump_vector = Vector3(0, 0, 0)
			posz = 0
		else:
			posz += delta_jump
			jump_vector.z += Z_ACCEL * delta
	
	if num == 1:
		world.set_p1pos(Vector3(posxy.x, posxy.y, posz))
	elif num == 2:
		world.set_p2pos(Vector3(posxy.x, posxy.y, posz))
	
	hits = get_node("/root/GameData").get_player_hits(num)
	
	time += delta
	
	update()

func set_num(n):
	if n <= 0:
		return
	num = n
	var pointmanager = get_node("/root/Point3DManager")
	var griddim = pointmanager.get_grid_dimension()
	if n == 1:
		posxy = Vector2(griddim.x - 2, griddim.y - 2)
		color = Color(1, 0, 0)
	if n == 2:
		posxy = Vector2(1, 1)
		color = Color(0, 0, 1)

func get_num():
	return num
	
var NOISE_SIZE = 100
	
func _draw():
	var respawn = get_node("/root/GameData").is_player_respawn(num)
	
	rotation = (time - int(time)) * 360 * ROTATION_SPEED
	
	var respawnhide = false
	if respawn && (int(time * 8) % 2) == 0:
		respawnhide = true
	
	var man3d = get_node("/root/Point3DManager")
	var pos = Vector3(posxy.x, posxy.y, posz)
	var scale = man3d.get_scale(pos)
	var pos_screen = man3d.project_point(pos)
	
	scale *= max((1.0 - 2 * kill_anim / KILL_ANIM_DURATION), 0)
	
	var size = man3d.get_tile_size() * 0.5
	
	var localcolor = color
	
	var inner_radius = 0.0
	if hits == 2:
		inner_radius = 0.4
	elif hits == 1:
		inner_radius = 0.7
	
	if !respawnhide:
		draw_circle_full_poly(pos_screen, size * scale, size * scale * inner_radius, \
			rotation, Color(0, 0, 0))
		draw_circle_full(pos_screen, size * scale, rotation, localcolor)
		if inner_radius > 0:
			draw_circle_full(pos_screen, size * scale * inner_radius, rotation, localcolor)
var NUM_CIRCLE_SEGMENTS = 6

func draw_circle_full( center, radius, offset, color ):
    var nbPoints = NUM_CIRCLE_SEGMENTS
    var pointsArc = Vector2Array()
    
    for i in range(nbPoints+1):
        var anglePoint = i * 360 / nbPoints + rotation
        var point = center + Vector2( cos(deg2rad(anglePoint)), sin(deg2rad(anglePoint)) )* radius
        pointsArc.push_back( point )
    
    for indexPoint in range(nbPoints):
        draw_line(pointsArc[indexPoint], \
			pointsArc[indexPoint+1], color, line_size)
    pass

func draw_circle_full_poly( center, radius, inner, offset, color ):
	var nbPoints = NUM_CIRCLE_SEGMENTS
	var pointsArc = Vector2Array()
	var colors = ColorArray([color])
	
	if inner <= 0:
		for i in range(nbPoints):
			var anglePoint = i * 360 / nbPoints + rotation
			pointsArc.push_back(center + Vector2( cos( deg2rad(anglePoint) ), sin( deg2rad(anglePoint) ) )* radius)
		draw_polygon(pointsArc, colors)
	else:
		var innerPointsArc = Vector2Array()
		for i in range(nbPoints):
			var anglePoint = i * 360 / nbPoints + rotation
			var c = cos( deg2rad(anglePoint) )
			var s = sin( deg2rad(anglePoint) )
			pointsArc.push_back(center + Vector2(c, s)* radius)
			innerPointsArc.push_back(center + Vector2(c, s)* inner)
		var poly1 = Vector2Array()
		var poly2 = Vector2Array()
		for i in range(nbPoints / 2 + 1):
			poly1.push_back(pointsArc[i])
		for i in range(nbPoints / 2 + 1):
			poly1.push_back(innerPointsArc[nbPoints / 2 - i])
		poly2.push_back(pointsArc[0])
		for i in range(nbPoints / 2 + 1):
			poly2.push_back(pointsArc[nbPoints - i - 1])
		for i in range(nbPoints / 2 + 1):
			poly2.push_back(innerPointsArc[nbPoints - (nbPoints / 2 - i) - 1])
		poly2.push_back(innerPointsArc[0])
		draw_polygon(poly1, colors)
		draw_polygon(poly2, colors)