extends Node

export var x = 0
export var y = 0
var size = 0.9

var flipping = 0
var direction_flip = Vector2(0, 0)
var flipped_by_player = 0

var DIRECTION_CLASSIFIER = 0.3

var NSIZE = 0.7071

var MORTAL_THRESHOLD = 0.2

var line_size = 1

var void = false

var voidable = false
var to_void = false
var to_void_time = 0

var TO_VOID_FADEOUT_TIME = 1

func _ready():
	line_size = get_node("/root/Point3DManager").get_line_size()
	line_size = 1
	set_process(true)

func _process(delta):
	if get_node("/root/GameData").is_someone_dead():
		return
	
	if void:
		kill_player()
		return
	
	var mark_update = false
	
	var waveManager = get_node("/root/Waves")
	var waves = waveManager.get_waves()
	var cur_time = waveManager.get_time()
	var maxh = 0
	var direction = Vector2()
	var player = 0
	for wave in waves:
		var direction_wrap = Vector2(x, y) - Vector2(wave.x, wave.y)
		var distance = direction_wrap.length()
		if distance < 0.71:
			continue
		var dist_to_front = wave.curfront - distance
		var h = min(max(wave.size - abs(dist_to_front), 0), 1)
		if h > maxh:
			maxh = h
			direction = direction_wrap
			player = wave.source_player
			
	if flipping != maxh:
		flipping = maxh
		direction = direction.normalized()
		if direction.x > DIRECTION_CLASSIFIER:
			direction_flip.x = 1
		elif direction.x < -DIRECTION_CLASSIFIER:
			direction_flip.x = -1
		if direction.y > DIRECTION_CLASSIFIER:
			direction_flip.y = 1
		elif direction.y < -DIRECTION_CLASSIFIER:
			direction_flip.y = -1
		direction_flip = direction_flip.normalized()
		if flipping == 0:
			flipped_by_player = 0
		else:
			flipped_by_player = player
		mark_update = true
		
	if voidable && !to_void:
		var world = get_node("/root/World")
		var p1_jumped = world.get_p_jumped(1)
		var p2_jumped = world.get_p_jumped(2)
		if ((p1_jumped.x == x && p1_jumped.y == y) ||
			(p2_jumped.x == x && p2_jumped.y == y)):
			to_void = true
		
	if to_void && !void:
		to_void_time += delta
		mark_update = true
		
	if to_void && !void && to_void_time > TO_VOID_FADEOUT_TIME:
		get_node("/root/World").notify_new_void_tile(Vector2(x, y))
		void = true
		
	if mark_update:
		update()
		
	if flipping >= MORTAL_THRESHOLD:
		kill_player()

func kill_player():
	var world = get_node("/root/World")
	var p1_pos = world.get_p1pos()
	var p2_pos = world.get_p2pos()
	if p1_pos.x == x && \
		p1_pos.y == y && \
		p1_pos.z <= 0:
		get_node("/root/GameData").kill_p1()
	if p2_pos.x == x && \
		p2_pos.y == y && \
		p2_pos.z <= 0:
		get_node("/root/GameData").kill_p2()

var RADIO_ACTIVE_COLOR = Color(1, 0.7, 0)

func _draw():
	var man3d = get_node("/root/Point3DManager")
	if void || to_void:
		var sizey = size * 0.5
		var coordinates = [Vector2(0.3, 0.1),
			Vector2(0.7, 0.8),
			Vector2(0.3, 0.8),
			Vector2(0.7, 0.1),
			Vector2(0.9, 0.4),
			Vector2(0.1, 0.4)]
		var points = []
		for c in coordinates:
			points.push_back(Vector2( \
				x - (c.x - 0.5) * size, y - (c.y - 0.5) * size))
		var projectedpoints = []
		for pi in range(points.size()):
			var toproject = Vector3(points[pi].x, points[pi].y, 0)
			projectedpoints.push_back(man3d.project_point(toproject))
			
		var color = RADIO_ACTIVE_COLOR
		if !void:
			var progress = to_void_time / TO_VOID_FADEOUT_TIME
			color.r *= progress
			color.g *= progress
			color.b *= progress
		for j in range(points.size()):
			var p1 = j
			var p2 = j + 1
			if j == (points.size() - 1):
				p2 = 0
			draw_line(projectedpoints[p1], projectedpoints[p2], color, line_size)
		if void:
			return
	
	var points = [Vector2(), Vector2(), Vector2(), Vector2()]
	
	if direction_flip.y == 0:
		var sizex = size * 0.5 * (1.0 - flipping)
		var sizey = size * 0.5
		points = [Vector2(x - sizex, y - sizey),
				Vector2(x - sizex, y + sizey),
				Vector2(x + sizex, y + sizey),
				Vector2(x + sizex, y - sizey)]
	elif direction_flip.x == 0:
		var sizex = size * 0.5
		var sizey = size * 0.5 * (1.0 - flipping)
		points = [Vector2(x - sizex, y - sizey),
				Vector2(x - sizex, y + sizey),
				Vector2(x + sizex, y + sizey),
				Vector2(x + sizex, y - sizey)]
	else:
		var to_center = [Vector2(-NSIZE, -NSIZE),
					Vector2(-NSIZE, NSIZE),
					Vector2(NSIZE, NSIZE),
					Vector2(NSIZE, -NSIZE)]
		var sizes = [size, size, size, size]
		
		for tci in range(to_center.size()):
			var tc = to_center[tci]
			var dot = abs(tc.dot(direction_flip))
			sizes[tci] *= (1.0 - dot * flipping)
		
		points = [Vector2(x - sizes[0] * 0.5, y - sizes[0] * 0.5),
					Vector2(x - sizes[1] * 0.5, y + sizes[1] * 0.5),
					Vector2(x + sizes[2] * 0.5, y + sizes[2] * 0.5),
					Vector2(x + sizes[3] * 0.5, y - sizes[3] * 0.5)]
	
				
	var projectedpoints = [Vector2(), Vector2(), Vector2(), Vector2()]
	for pi in range(points.size()):
		var toproject = Vector3(points[pi].x, points[pi].y, 0)
		projectedpoints[pi] = man3d.project_point(toproject)
		
	var color = Color(0, 0.8, 0)
	if get_node("/root/GameData").is_level_icy():
		color = Color(0, 0.6, 0.4)
	if flipped_by_player == 1:
		color.r = color.r * (1.0 - flipping) + flipping
		color.g = color.g * (1.0 - flipping)
		color.b = color.b * (1.0 - flipping)
	elif flipped_by_player == 2:
		color.r = color.r * (1.0 - flipping)
		color.g = color.g * (1.0 - flipping)
		color.b = color.b * (1.0 - flipping) + flipping
		
	if to_void && !void:
		var progress = 1.0 - to_void_time / TO_VOID_FADEOUT_TIME
		color.r *= progress
		color.g *= progress
		color.b *= progress
		
	draw_line(projectedpoints[0], projectedpoints[1], color, line_size)
	draw_line(projectedpoints[1], projectedpoints[2], color, line_size)
	draw_line(projectedpoints[2], projectedpoints[3], color, line_size)
	draw_line(projectedpoints[3], projectedpoints[0], color, line_size)

func set_void(v):
	void = v
	
func set_voidable(v):
	voidable = v